﻿using System;
using System.Text.RegularExpressions;
using System.Threading;
using ZennoLab.CommandCenter;

namespace SmsService.Services
{
    public class SmsActivateRent : SmsHub
    {
        public Rent RentTime { get; set; }

        public SmsActivateRent(string token, Rent rentTime) : base(token)
        {
            Name = "Sms-activate Аренда";
            ApiHandler = "https://api.sms-activate.org/stubs/handler_api.php";
            Token = token;
            RentTime = rentTime;
        }

        /// <summary>
        /// Получить номер для Аренды.
        /// </summary>
        /// <param name="service">Сервис</param>
        /// <param name="country">Страна</param>
        /// <param name="smsOperator">Оператор</param>
        public override bool GetNumber(string service, string country, string smsOperator = "any")
        {
            var reqContent = $"api_key={Token}" +
                 "&action=getRentNumber" +
                 $"&rent_time={RentTime}" +
                 $"&service={service}" +
                 $"&country={country}" +
                 $"&operator={smsOperator}";

            Responce = ZennoPoster.HttpPost(ApiHandler, reqContent, proxy: Proxy);

            if (!Responce.Contains("success"))
                throw new Exception($"Ошибка получения \"Номера\".\t{Responce}");

            var jResp = Global.ZennoLab.Json.Linq.JObject.Parse(Responce);

            var number = jResp["phone"].SelectToken("number").ToString();
            var id = jResp["phone"].SelectToken("id").ToString();

            if (string.IsNullOrWhiteSpace(number) || string.IsNullOrWhiteSpace(id))
            {
                return false;
            }

            Number = number;
            Id = id;

            return true;
        }

        /// <summary>
        /// Получаем Код.
        /// </summary>
        /// <param name="attempts">Количество попыток получения смс</param>
        /// <param name="pause_Sec">Пауза между запросами Кода (секунды)</param>
        /// <returns></returns>
        public override bool GetCode(int attempts = 3, int pause_Sec = 30)
        {
            //Ловим код
            for (int i = 0; i < attempts; i++)
            {
                Thread.Sleep(pause_Sec * 1000);
                if (GetStatus())
                {
                    return true;
                }
            }

            //Отмена номера, если смс не получили
            SmsCancel();
            return false;
        }

        /// <summary>
        /// Получить статус.
        /// </summary>
        public override bool GetStatus()
        {
            var id = Id;
            if (string.IsNullOrEmpty(id))
                throw new Exception("Sms id пустой!");

            var reqContent = $"api_key={Token}" +
                 "&action=getRentStatus" +
                 $"&id={id}";

            Responce = ZennoPoster.HttpPost(ApiHandler, reqContent, proxy: Proxy);

            if (Responce.Contains("success"))
            {
                var jResp = Global.ZennoLab.Json.Linq.JObject.Parse(Responce);
                var smsText = jResp["values"]["0"].SelectToken("text").ToString();
                var code = Regex.Match(smsText, "[\\-\\d]+").ToString();

                if (string.IsNullOrWhiteSpace(code))
                    return false;

                Code = code;
                return true;
            }
            return false;
        }

        /// <summary>
        /// Установить статус.
        /// </summary>
        /// <param name="status">Статус</param>
        public override string SetSatus(SmsStatus status)
        {
            var id = Id;
            if (string.IsNullOrEmpty(id))
                throw new Exception("Sms id пустой!");

            var reqContent = $"api_key={Token}" +
                 "&action=setRentStatus" +
                 $"&status={(int)status}" +
                 $"&id={id}";

            Responce = ZennoPoster.HttpPost(ApiHandler, reqContent, proxy: Proxy);

            return Responce;
        }
    }
}
