﻿using SmsService.Interfaces;
using System;
using System.Threading;
using ZennoLab.CommandCenter;

namespace SmsService.Services
{
    public class SmsHub : ISmsService
    {
        public string Id { get; protected set; }
        public string Name { get; protected set; }
        public string Code { get; protected set; }
        public string Number { get; protected set; }
        public string Responce { get; protected set; }

        public string Proxy { get; protected set; }
        public string Token { get; protected set; }
        public string ApiHandler { get; protected set; }

        public SmsHub(string token)
        {
            Name = "SmsHub";
            ApiHandler = "https://smshub.org/stubs/handler_api.php";
            Token = token;
        }

        /// <summary>
        /// Установить прокси.
        /// </summary>
        /// <param name="proxy">proxy: login:pass@ip:port</param>
        public void SetProxy(string proxy)
        {
            Proxy = proxy;
        }

        /// <summary>
        /// Получить баланс.
        /// </summary>
        public virtual decimal GetBalance()
        {
            var reqContent = $"api_key={Token}" +
                "&action=getBalance";

            Responce = ZennoPoster.HttpPost(ApiHandler, reqContent, proxy: Proxy);
            var balance = decimal.Parse(Responce.Split(':')[1].Replace(".", ","));
            return balance;
        }

        /// <summary>
        /// Получить номер.
        /// </summary>
        /// <param name="service">Сервис</param>
        /// <param name="country">Страна</param>
        /// <param name="smsOperator">Оператор</param>
        public virtual bool GetNumber(string service, string country, string smsOperator = "any")
        {
            var reqContent = $"api_key={Token}" +
                 "&action=getNumber" +
                 $"&service={service}" +
                 $"&country={country}" +
                 $"&operator={smsOperator}";

            Responce = ZennoPoster.HttpPost(ApiHandler, reqContent, proxy: Proxy);

            if (!Responce.Contains("ACCESS_NUMBER"))
                throw new Exception($"Ошибка получения \"Номера\".\t{Responce}");

            var number = Responce.Split(':')[2];
            var id = Responce.Split(':')[1];

            if (string.IsNullOrWhiteSpace(number) || string.IsNullOrWhiteSpace(id))
            {
                return false;
            }

            Number = number;
            Id = id;

            return true;
        }

        /// <summary>
        /// Получаем Код.
        /// </summary>
        /// <param name="attempts">Количество попыток получения смс</param>
        /// <param name="pause_Sec">Пауза между запросами Кода (секунды)</param>
        /// <returns></returns>
        public virtual bool GetCode(int attempts = 3, int pause_Sec = 30)
        {
            //Ловим код
            for (int i = 0; i < attempts; i++)
            {
                Thread.Sleep(pause_Sec * 1000);
                if (GetStatus())
                {
                    return true;
                }
            }

            //Отмена номера, если смс не получили
            SmsCancel();
            return false;
        }

        /// <summary>
        /// Получить статус.
        /// </summary>
        public virtual bool GetStatus()
        {
            if (string.IsNullOrEmpty(Id))
                throw new Exception("Sms id пустой!");

            var reqContent = $"api_key={Token}" +
                 "&action=getStatus" +
                 $"&id={Id}";

            Responce = ZennoPoster.HttpPost(ApiHandler, reqContent, proxy: Proxy);

            var smsStatus = Responce.Split('_')[1];

            if (smsStatus.Contains("OK"))
            {
                Code = Responce.Split(':')[1];
                return true;
            }
            return false;
        }

        /// <summary>
        /// Статус номера Отмена.
        /// </summary>
        public void SmsCancel() => SetSatus(SmsStatus.Cancel);

        /// <summary>
        /// Статус номера ОК.
        /// </summary>
        public void SmsOk() => SetSatus(SmsStatus.OK);

        /// <summary>
        /// Установить статус.
        /// </summary>
        /// <param name="status"></param>
        public virtual string SetSatus(SmsStatus status)
        {
            if (string.IsNullOrEmpty(Id))
                throw new Exception("Sms id пустой!");

            var reqContent = $"api_key={Token}" +
                 "&action=setStatus" +
                 $"&status={(int)status}" +
                 $"&id={Id}";

            Responce = ZennoPoster.HttpPost(ApiHandler, reqContent, proxy: Proxy);

            return Responce;
        }
    }
}
